# An alternative to Capybara's built-in implicit waits (using the 'find' method)
# @author nredmond@masteryconnect.com
module ImplicitWaitManager
  # An exception to be thrown when an element was not found in the given amount of time.
  class PageNotLoadedException < StandardError
    def initialize message
      super message
    end
  end

  # Waits for an element to be visible on the page. Yields to a block where 
  # the element can be queried.
  # @param seconds [Integer] The maximum number of seconds to wait for the 
  #   element to become visible
  # @param interval [Integer] The interval, in seconds, for which the function 
  #   should wait to try again when an element is not visible
  # @param message [String] The error message to present when the element was 
  #   not visible within the specified number of seconds
  def wait_until(seconds = 10, interval = 1,
    message = 'The action could not be completed.' + 
      ' Maybe the page did not load, or the wrong locator was used.')
    max_load_attempts = seconds / interval
    number_load_attempts = 0

    begin
      yield
    rescue Capybara::ElementNotFound
      number_load_attempts += 1

      if number_load_attempts > max_load_attempts
        raise PageNotLoadedException.new message
      else
        sleep interval
        retry
      end
    end
  end

  def finished_jquery_requests?
    evaluate_script('(typeof jQuery === "undefined") || (jQuery.active == 0)')
  end

  def finished_angular_requests?
    evaluate_script('(typeof angular === "undefined") || (angular.element(".ng-scope").injector().get("$http").pendingRequests.length == 0)')
  end

  def no_animations?
    evaluate_script('(typeof $ === "undefined") || ($(":animated").length == 0)')
  end

  def no_overlays?
    evaluate_script('(typeof jQuery === "undefined") || (jQuery(".blockUI").length == 0)')
  end

  def all_ajax_is_loaded?
    if no_overlays? && no_animations? && finished_angular_requests? && finished_jquery_requests?
      true
    end
  end

  def wait_for_ajax
    Timeout.timeout(Capybara.default_max_wait_time) do
      loop until all_ajax_is_loaded?
    end
  end
  
end